#include "MESH_DFSCoreWalker.h"
#include "MESH_X3DGroupingNode.h"
#include "MESH_X3DMeshNode.h"
#include "MESH_Shape.h"
#include "X3D_X3DNode.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace MESH {

DFSCoreWalker::DFSCoreWalker()
: X3DComponentWalker()
{
  defineComponentName("Core", "MESH");
  
  defineWalkingFunction(&DFSCoreWalker::walkX3DGroupingNode);
  defineWalkingFunction(&DFSCoreWalker::walkX3DMeshNode);
  defineWalkingFunction(&DFSCoreWalker::walkShape);
}

DFSCoreWalker::~DFSCoreWalker()
{
}

void DFSCoreWalker::walkX3DGroupingNode(X3DGroupingNode *N) const
{
  visitor->enter(N);
  
  MFNode children = N->getChildren();
  for (MFNode::iterator it = children.begin(); it != children.end(); ++it)
  {  
    if ((*it != 0) && (visitor->walkOn(N, *it)))
      walker->walk(*it);
  }    
  
  X3D::MFNode x3dChildren = N->getX3DChildren();
  for (X3D::MFNode::iterator it = x3dChildren.begin(); it != x3dChildren.end(); ++it)
  {  
    if ((*it != 0) && (visitor->walkOn(N, *it)))
      walker->walk(*it);
  }
  
  visitor->leave(N);
}

void DFSCoreWalker::walkX3DMeshNode(X3DMeshNode *N) const
{
  visitor->enter(N);
  
  SFNode C = N->getVertex();
  if ((C != 0) && (visitor->walkOn(N, C)))
    walker->walk(C);
  
  visitor->leave(N);
}

void DFSCoreWalker::walkShape(Shape *S) const
{
  visitor->enter(S);
  
  SFNode C = S->getMesh();
  if ((C != 0) && (visitor->walkOn(S, C)))
    walker->walk(C);
  
  X3D::SFNode CA = S->getAppearance();
  if ((CA != 0) && (visitor->walkOn(S, CA)))
    walker->walk(CA);
  
  visitor->leave(S);
}


}
}
