#include "X3D_LineProperties.h"

namespace X3DTK {
namespace X3D {

LineProperties::LineProperties()
: X3DAppearanceChildNode(), _lineStyle(0), _width(0.0f)
{
  defineTypeName("LineProperties", "Shape");
}

LineProperties::LineProperties(const SFInt32 &lineStyle, const SFFloat &width)
: X3DAppearanceChildNode(), _lineStyle(lineStyle), _width(width)
{
  defineTypeName("LineProperties", "Shape");
}

LineProperties::LineProperties(const LineProperties &L)
: X3DAppearanceChildNode(L), _lineStyle(L._lineStyle), _width(L._width)
{
}

SFAbstractNode LineProperties::clone() const
{
  return new LineProperties(*this);
}

LineProperties::~LineProperties()
{
}

void LineProperties::setLineStyle(const SFInt32 &lineStyle)
{
  _lineStyle = lineStyle;
}

void LineProperties::setWidth(const SFFloat &width)
{
  _width = width;
}

void LineProperties::load(const X3DFileElement *element)
{
  int index;
  index = element->getIndexAttribute("lineStyle");
  if (index != -1)
    _lineStyle = element->getAttribute(index).toInt();
   
  index = element->getIndexAttribute("width");
  if (index != -1)
    _width = element->getAttribute(index).toFloat(); 
}

SFString &LineProperties::write(SFString &output) const
{
  if (_lineStyle != 0)
    output += " lineStyle = \"" + toSFString(_lineStyle) + "\"";  
  if (_width != 0.0f)
    output += " width = \"" + toSFString(_width) + "\"";
      
  return output;
}

}
}
