#include "X3D_MeshBuilderGroupingVisitor.h"
#include "X3D_X3DGroupingNode.h"
#include "X3D_StaticGroup.h"
#include "X3D_Transform.h"
#include "X3D_Switch.h"
#include "MESH_Transform.h"
#include "MESH_Switch.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

MeshBuilderGroupingVisitor::MeshBuilderGroupingVisitor()
: GroupingVisitor()
{
  // Enter functions.
  defineEnterFunction(&MeshBuilderGroupingVisitor::enterX3DChildNode);
  defineEnterFunction(&MeshBuilderGroupingVisitor::enterX3DGroupingNode);
  defineEnterFunction(&MeshBuilderGroupingVisitor::enterStaticGroup);
  defineEnterFunction(&MeshBuilderGroupingVisitor::enterTransform);
  defineEnterFunction(&MeshBuilderGroupingVisitor::enterSwitch);

  defineLeaveFunction(&MeshBuilderGroupingVisitor::leaveX3DChildNode);
  defineLeaveFunction(&MeshBuilderGroupingVisitor::leaveX3DGroupingNode);
  
  // StateVariables assignation.
  stateVariables = GraphTraversal::getInstanceOf<MeshBuilderStateVariables>();
}

MeshBuilderGroupingVisitor::~MeshBuilderGroupingVisitor()
{
}

void MeshBuilderGroupingVisitor::enterX3DChildNode(X3DChildNode *N) const
{
  stateVariables->disableChildrenVisit();
  X3DNode *NN = static_cast<X3DNode *>(stateVariables->clone(N));
  
  MESH::X3DGroupingNode *G = dynamic_cast<MESH::X3DGroupingNode *>(stateVariables->getTop());
  if (G != 0)
    G->addChild(NN);
}

void MeshBuilderGroupingVisitor::enterX3DGroupingNode(X3DGroupingNode *N) const
{
  MESH::SFNode MN = stateVariables->getNode(N);
  if (MN == 0)
  {
    MN = new MESH::Transform();
    MN->setName(N->getName());
      
    stateVariables->addCoupleNode(N, MN);
  }
  stateVariables->pushNode(MN);
}

void MeshBuilderGroupingVisitor::enterStaticGroup(StaticGroup *S) const
{
  MESH::SFNode MS = stateVariables->getNode(S);
  if (MS == 0)
  {
    MS = new MESH::Transform();  
    MS->setName(S->getName());
    
    stateVariables->addCoupleNode(S, MS);
  }
  stateVariables->pushNode(MS);
}

void MeshBuilderGroupingVisitor::enterTransform(Transform *T) const
{
  MESH::Transform *MT = dynamic_cast<MESH::Transform *>(stateVariables->getNode(T));
  if (MT == 0)
  {
    MT = new MESH::Transform();
    MT->setName(T->getName());    
    
    MT->setCenter(T->getCenter());
    MT->setRotation(T->getRotation());
    MT->setScale(T->getScale());
    MT->setScaleOrientation(T->getScaleOrientation());
    MT->setTranslation(T->getTranslation());
      
    stateVariables->addCoupleNode(T, MT);
  }
  stateVariables->pushNode(MT);
}

void MeshBuilderGroupingVisitor::enterSwitch(Switch *S) const
{
  MESH::SFNode MS = stateVariables->getNode(S);
  if (MS == 0)
  {
    MS = new MESH::Switch();  
    MS->setName(S->getName());
        
    stateVariables->addCoupleNode(S, MS);
  }
  stateVariables->pushNode(MS);
}

void MeshBuilderGroupingVisitor::leaveX3DChildNode(X3DChildNode *) const
{
  stateVariables->enableChildrenVisit();
}

void MeshBuilderGroupingVisitor::leaveX3DGroupingNode(X3DGroupingNode *) const
{
  stateVariables->popNode();
}

}
}
