#include "GraphTraversal.h"
#include "Visitor.h"
#include "Walker.h"

#include <iostream>

using namespace std;

namespace X3DTK {

list<StateVariables *> GraphTraversal::_stateVariablesList = list<StateVariables *>();

GraphTraversal::GraphTraversal()
: visitor(new Visitor()), walker(new Walker())
{
  walker->setVisitor(visitor);
}

GraphTraversal::~GraphTraversal()
{
  delete walker;
  delete visitor;
}

void GraphTraversal::setComponentWalker(X3DComponentWalker *component)
{
  walker->setComponentWalker(component);
}

void GraphTraversal::setComponentVisitor(X3DComponentVisitor *component)
{
  visitor->setComponentVisitor(component);
}

void GraphTraversal::setAutoDeleteComponents(bool value)
{
  visitor->setAutoDeleteComponents(value);
  walker->setAutoDeleteComponents(value);
}
  
void GraphTraversal::traverse(SFAbstractNode N) const
{
  if ((walker != 0) && (N != 0))
    walker->walk(N);
}

GraphTraversal *X3DTK::joinGraphTraversals(GraphTraversal *A1, GraphTraversal *A2)
{
  GraphTraversal *A = new GraphTraversal();
  
  A->visitor = joinVisitors(A1->visitor, A2->visitor);
  A->walker = joinWalkers(A1->walker, A2->walker);
  A->walker->setVisitor(A->visitor);
  
  return A;
}

GraphTraversal *X3DTK::mergeGraphTraversals(GraphTraversal *A1, GraphTraversal *A2)
{
  GraphTraversal *A = new GraphTraversal();
  
  delete A->visitor;
  delete A->walker;
  
  A->visitor = joinVisitors(A1->visitor, A2->visitor);
  A->walker = joinWalkers(A1->walker, A2->walker);
  A->walker->setVisitor(A->visitor);
  
  delete A1;
  delete A2;
  
  return A;
}

}
